@@ -30,18 +30,6 @@ C<< Bootloader::Tools::IsDMRaidSlave ($kernel_disk); >>
 
 C<< Bootloader::Tools::IsDMDevice($dev); >>
 
-C<< $dm_dev = Bootloader::Udev2DMDev($udevdevice); >>
-
-C<< $udev_dev = Bootloader::Tools::DMDev2Udev($dmdev); >>
-
-C<< $majmin = Bootloader::Tools::DMDev2MajMin($dmdev); >>
-
-C<< $majmin = Bootloader::Tools::Udev2MajMin($udev); >>
-
-C<< $udev_dev = Bootloader::Tools::MajMin2Udev($majmin); >>
-
-C<< $dm_dev = Bootloader::Tools::MajMin2DMDev($majmin); >>
-
 C<< $md_ref = Bootloader::Tools::ReadRAID1Arrays (); >>
 
 C<< $loader = Bootloader::Tools::GetBootloader (); >>
@@ -101,6 +89,7 @@ our @EXPORT = qw(InitLibrary CountImageSections CountSections
 use Bootloader::Library;
 use Bootloader::Core;
 use Bootloader::Path;
+use Bootloader::Logger;
 
 my $lib_ref = undef;
 my $dmsetup = undef;
@@ -126,7 +115,7 @@ sub DumpLog {
 	return strftime ( "%Y-%m-%d %H:%M:%S", localtime);
     }
 
-    foreach my $rec (@{$core_lib->GetLogRecords ()})
+    foreach my $rec (@{Bootloader::Logger::instance()->GetLogRecords()})
     {
 	my $message = $rec->{"message"};
 	my $level = $rec->{"level"};
@@ -151,7 +140,7 @@ sub DumpLog {
 
 	    # If writing to perl logfile, also print warnings to STDERR
 	    if ($using_logfile) {
-		print STDERR (timestamp() . " WARNING: $message\n");
+		print STDERR ("Perl-Bootloader: ".timestamp() . " WARNING: $message\n");
 	    }
 	}
 	elsif ($level eq "error")
@@ -160,7 +149,7 @@ sub DumpLog {
 
 	    # If writing to perl logfile, also print errors to STDERR
 	    if ($using_logfile) {
-		print STDERR (timestamp() . " ERROR: $message\n");
+		print STDERR ("Perl-Bootloader: ".timestamp() . " ERROR: $message\n");
 	    }
 	}
 	else
@@ -199,33 +188,26 @@ See InitLibrary function for example.
 =cut
 
 sub ReadMountPoints {
+    my $udevmap = shift;
     open (FILE, Bootloader::Path::Fstab()) || 
 	die ("ReadMountPoints(): Failed to open /etc/fstab");
 
     my %mountpoints = ();
     while (my $line = <FILE>)
     {
-	if ($line =~ /^[ \t]*([^ \t]+)[ \t]+([^ \t]+).*/)
+	if ($line =~ /^\s*(\S+)\s+(\S+).*/)
 	{
 	    my $dev = $1;
 	    my $mp = $2;
 	    if (substr ($dev, 0, 1) ne "#")
 	    {
-		if ($dev =~ m/^LABEL=/ || $dev =~ m/UUID=/)
+		if ($dev =~ m/^LABEL=(.*)/)
 		{
-                    my $command = Bootloader::Path::Blkid() . " -t $dev |";
-		    open (BLKID, $command) || 
-			die ("ReadMountPoints(): Failed to run blkid");
-
-		    my $line = <BLKID>;
-		    close (BLKID);
-		    chomp ($line);
-		    my $index = index ($line, ":");
-		    if ($index != -1)
-		    {
-			$dev = substr ($line, 0, $index);
-		    }
-		}
+                    $dev = "/dev/disk/by-label/$1"; #do not translate otherwise it changes root always bnc#575362
+                } elsif ($dev =~ m/UUID=(.*)/){
+                    $dev = "/dev/disk/by-uuid/$1";
+                }
+                $mp =~ s/\\040/ /; #handle spaces in fstab
 		$mountpoints{$mp} = $dev;
 	    }
 	}
@@ -234,45 +216,6 @@ sub ReadMountPoints {
     return \%mountpoints;
 }
 
-sub Udev2Dev {
-    my $udev = shift;
-
-    my $mounted = undef;
-    unless (-e "/sys/block/") {
-      $mounted = `mount /sys`;
-    }
-    # FIXME: maybe useless code  
-    my $cmd = "udevadm info -q name -p '/block/$udev'";
-    my $dev = qx{ $cmd 2>/dev/null };
-    chomp ($dev);
-
-    if ($dev ne "") {
-        $dev = "/dev/$dev";
-    }
-
-    # Fallback in case udevadm info fails
-    else {
-
-	#If $udev consists of both device and partition - e.g. "sda/sda1" -
-	#then only assign the partition.
-    	if ($udev =~ m#\w+/(\w+)#) {
-	    $dev = "/dev/$1";
-	}
-
-	#Else, just assign the content of $udev, e.g. "sda".
-	else {
-	    $dev = "/dev/$udev";
-	}
-    }
-
-    # CCISS maps slashes to bangs so we have to reverse that.
-    $dev =~ s:!:/:g;
-
-    `umount /sys` if (defined $mounted);
-
-    return $dev;
-}
-
 =item
 C<< $part_ref = Bootloader::Tools::ReadPartitions (); >>
 
@@ -285,28 +228,24 @@ See InitLibrary function for example.
 
 # FIXME: this has to be read through yast::storage
 sub ReadPartitions {
+    my $udevmap = shift;
     my $sb = "/sys/block";
     my $mounted = undef;
-    my $logname = Bootloader::Path::Logname();
+    my $logger = Bootloader::Logger::instance();
     unless (-e $sb) {
       $mounted = `mount /sys`;
-       open (LOG, ">>$logname");
-       print LOG ("Mount /sys\n");
-       close LOG;
+      $logger->milestone("ReadPartitions: Mount /sys");
     }
     opendir(BLOCK_DEVICES, "$sb") || 
 	die ("ReadPartitions(): Failed to open dir $sb");
 
     # get disk devices
     my @disks = grep {
-	!m/^\./ and -r "$sb/$_/range" and qx{ cat $sb/$_/range } > 1
+	!m/^\./ && (( -r "$sb/$_/ext_range" and qx{ cat $sb/$_/ext_range } > 1) || ( -r "$sb/$_/range" and qx{ cat $sb/$_/range } > 1))
     } readdir(BLOCK_DEVICES);
     closedir BLOCK_DEVICES;
 
-    open (LOG, ">>$logname");
-    print LOG ("Finded disks: ". join (",",@disks)||"");
-    print LOG ("\n");
-    close LOG;
+    $logger->milestone("ReadPartitions: Finded disks: ". join (",",@disks));
 
     # get partition info for all partitions on all @disks
     my @devices = ();
@@ -320,51 +259,8 @@ sub ReadPartitions {
 
     foreach my $disk (@disks)
     {
-	# get kernel device name for this device
-	my $dev_disk = Udev2Dev ($disk);
-
-	# get additional info from parted for the partitions on this $disk
-	#
-	# Disk geometry for /dev/sdb: 0cyl - 14593cyl
-	# BIOS cylinder,head,sector geometry: 14593,255,63.  Each cylinder is 8225kB.
-	# Disk label type: msdos
-	# Number  Start   End     Size    Type      File system  Flags
-	# 1       0cyl    63cyl   63cyl   primary   reiserfs     raid, type=fd
-	# 2       64cyl   2022cyl 1959cyl primary   ext3         boot, raid, type=fd
-	# 3       2023cyl 3067cyl 1045cyl primary   linux-swap   raid, type=fd
-	# 4       3068cyl 14592cyl 11525cyl extended               lba, type=0f
-	# 5       3068cyl 3198cyl 130cyl  logical   reiserfs     type=83
-	#
-	my @parted_info_list = ();
-	my $parted_info = `parted -s $dev_disk unit cyl print`;
-        chomp $parted_info;
-	if ( $? == 0 ) {
-	    my @parted_info_split = split(/\n/, $parted_info);
-	    # skip header lines
-
-
-	    while ($#parted_info_split >=0 and $parted_info_split[0] !~ /^\s*\d/) {
-		shift @parted_info_split;
-	    }
-
-	    foreach $parted_info (@parted_info_split) {
-		# FIXME: parse the rest of the lines
-		chomp($parted_info);
-		my ($p_nr, $p_startcyl, $p_endcyl, $p_sizecyl, $p_type, $rest) =
-		    split(' ', $parted_info);
-
-		$p_startcyl =~ s/cyl$//;
-		$p_endcyl   =~ s/cyl$//;
-		$p_sizecyl  =~ s/cyl$//;
-		$p_type	    =~ s/^/`/;
-
-		# array: part_nr -> (startcyl, endcyl, sizecyl, type)
-		$parted_info_list[$p_nr] =
-		    [$p_startcyl, $p_endcyl, $p_sizecyl, $p_type];
-	    }
-	}
-
-        if (!IsDMDevice($disk) && !IsDMRaidSlave($disk)){
+        my $dev_disk = "/dev/$disk";
+        if (!IsDMDevice($dev_disk) && !IsDMRaidSlave($dev_disk)){
 	    # get partitions of $disk
 	    opendir(BLOCK_DEVICES, "$sb/$disk") ||
 	        die ("ReadPartitions(): Failed to open dir $sb/$disk");
@@ -374,18 +270,18 @@ sub ReadPartitions {
 	    } readdir (BLOCK_DEVICES);
 	    closedir BLOCK_DEVICES;
 
-            open (LOG, ">>$logname");
-            print LOG ("Finded parts: ". join (",",@parts)||"");
-            close LOG;
+            $logger->milestone("ReadPartitions: Finded parts: ". join (",",@parts));
 
 	    # generate proper device names and other info for all @part[ition]s
+      #raid have ! in names for /dev/raid/name (bnc#607852)
+      $dev_disk =~ s:!:/:;
 	    foreach my $part (@parts)
 	    {
 	        chomp ($part);
-	        $part = Udev2Dev ("$disk/$part");
-                open (LOG, ">>$logname");
-                print LOG ("Processing part: ".(join (",",$part)||"")."\n");
-                close LOG;
+	        $part = "/dev/$part";
+          #raid have ! in names for /dev/raid/name
+          $part =~ s:!:/:;
+          $part = $udevmap->{$part} if (defined $udevmap->{$part});
 
 	        my $index = substr ($part, length ($dev_disk));
 	        while (length ($index) > 0 && substr ($index, 0, 1) !~ /[0-9]/)
@@ -399,19 +295,13 @@ sub ReadPartitions {
 		#  0    device	    /dev/sda9
 		#  1    disk	    /dev/sda
 		#  2    nr		    9
-		#  3    fsid	    258
-		#  4    fstype	    Apple_HFS
-		#  5    part_type	    `primary
-		#  6    start_cyl	    0
-		#  7    size_cyl	    18237
+		#  3    fsid	    258 (not needed for update)
+		#  4    fstype	    Apple_HFS(not needed for update)
+		#  5    part_type	    `primary(not needed for update)
+		#  6    start_cyl	    0(not needed for update)
+		#  7    size_cyl	    18237(not needed for update)
 
-		push @devices, [$part, $dev_disk, $index, 0, "",
-		    defined( $parted_info_list[$index]->[3] ) ?
-			$parted_info_list[$index]->[3] : "",
-		    defined( $parted_info_list[$index]->[0] ) ?
-			$parted_info_list[$index]->[0] : 0,
-		    defined( $parted_info_list[$index]->[1] ) ?
-			$parted_info_list[$index]->[1] : 0];
+		push @devices, [$part, $dev_disk, $index, 0, "", "","",""];
             }
 	}
     }
@@ -431,16 +321,26 @@ Gets multipath configuration. Return reference to hash map, empty if system does
 =cut
 
 sub GetMultipath {
-  my %ret = {};
+  my %ret = ();
+  my $logger = Bootloader::Logger::instance();
+
+  unless (DMRaidAvailable())
+  {
+    return \%ret;
+  }
 
   $multipath = AddPathToExecutable("multipath");
 
   if (-e $multipath){
     my $command = "$multipath -d -l";
- #FIXME log output 
     my @result = qx/$command/;
     # return if problems occurs...typical is not loaded kernel module
     if ( $? ) {
+      if ($result[0] =~ m/kernel driver not loaded/) {
+        $logger->milestone("Tools::GetMultipath: multipath kernel module is not loaded Ecode:$?.");
+      } else {
+        $logger->warning("Tools::GetMultipath: multipath command failed with $?.");
+      }
       return \%ret;
     }
 
@@ -449,6 +349,7 @@ sub GetMultipath {
     my $line = "";
     $line = shift @result if (scalar @result != 0);
     while (scalar @result != 0){
+      $logger->milestone("Tools::GetMultipath: processing line $line");
       if ($line !~ m/^(\S+)\s.*dm-\d+.*$/){
         $line = shift @result;
         next;
@@ -456,18 +357,86 @@ sub GetMultipath {
       my $multipathdev = "/dev/mapper/$1";
       while (scalar @result != 0){
         $line = shift @result;
+        chomp $line;
+        $logger->milestone("Tools::GetMultipath: processing line $line");
         if ($line =~ m/(.*)dm-.*$/){
           last;
         }
         if ($line =~ m/\d+:\d+:\d+:\d+\s+(\S+)\s+/){
           $ret{"/dev/$1"} = $multipathdev;
+          $logger->milestone("Tools::GetMultipath: added /dev/$1 -> $multipathdev");
         }
       }
     }
   }
+  else
+  {
+    $logger->milestone("Tools::GetMultipath: multipath command not installed");
+  }
+
   return \%ret;
 }
 
+=item
+C<< Bootloader::Tools::GetMultipath (); >>
+
+Gets multipath configuration. Return reference to hash map, empty if system doesn't contain multipath.
+
+=cut
+
+sub GetUdevMapping {
+  my %mapping= ();
+
+  my @output = `find -P /dev -type b`;
+  chomp @output;
+  my $logger = Bootloader::Logger::instance();
+  for my $dev (@output) {
+    next if ($dev =~ m:^/dev/mapper/:);
+
+    my @output2 = `udevadm info -q all -n $dev 2>/dev/null`;
+    chomp @output2;
+
+    if ($dev =~ m:^/dev/dm:) #workaround for incosistency of device mapper and udev
+    {
+      my $dmdev = undef;
+      my $dmpart = undef;
+      for my $line (@output2)
+      {
+        if ($line =~ m/DM_NAME=(.*)$/)
+        {
+          $dmdev = $1;
+        }
+        elsif ($line =~ m/DM_PART=(.*)$/)
+        {
+          $dmpart = $1;
+        }
+      }
+
+      $logger->error("UDEVMAPPING: dmdev $dev doesn't have defined DM_NAME in udev") unless defined $dmdev;
+      my $prevdev = $dev;
+      $dev = "/dev/mapper/$dmdev";
+      # DM_NAME could contain also part in some case (bnc#590637)
+      $dev = $dev."_part$dmpart" if (defined $dmpart and $dev !~ /_part[0-9]+$/);
+      $mapping{$prevdev} = $dev; #maps also dm dev to device mapper
+    } #end of workaround
+
+    for my $line (@output2)
+    {
+      if ($line =~ m/S:\s(.*)$/)
+      {
+        $mapping{"/dev/$1"} = $dev;
+      }
+    }
+  }
+
+  while (my ($k,$v) = each (%mapping)){
+      $logger->milestone ("UDEV MAPPING: ".($k||"")." -> ".($v||""));
+  }
+
+
+  return \%mapping;
+}
+
 =item
 C<< $numDM = Bootloader::Tools::DMRaidAvailable (); >>
 
@@ -479,8 +448,8 @@ Return 0 if no device, 1 if there are any.
 sub DMRaidAvailable {
     my $retval = 0;
 
-    my $logname = Bootloader::Path::Logname();
-    qx{cat /proc/misc >> $logname};
+    my $logger = Bootloader::Logger::instance();
+    $logger->milestone(`cat /proc/misc`);
 
     # Check if device-mapper is available in /proc/misc
     my $dm_available = qx{grep device-mapper /proc/misc};
@@ -499,8 +468,8 @@ sub DMRaidAvailable {
 	$retval = $dm_devices ne "No devices found";
     }
     else {
-	print ("The command \"dmsetup\" is not available.\n");
-	print ("Is the package \"device-mapper\" installed?\n");
+	$logger->error("The command \"dmsetup\" is not available.");
+	$logger->error("Is the package \"device-mapper\" installed?");
     }
     
     return $retval;
@@ -513,44 +482,38 @@ reads partitions belonging to a Devicemapper RAID device.
 needed to be able to put get the correct translation
 into Grub notation
 DMRaid Devices look like:
-dmraid-<strange name>
+<strange name>
 
 DMRaid Partitions look like:
-partX-dmraid-<strange name>
+<strange name>_part\d
 
 
 =cut
 
-#FIXME: this has to be done through Storage
 sub ReadDMRaidPartitions {
 
     my @dmdisks = ();
     my @dmparts = ();
-    my $dmdev;
 
-    my $logname = Bootloader::Path::Logname();
+    my $logger = Bootloader::Logger::instance();
 
     open(DMDEV, "$dmsetup info -c --noheadings -o name |") || 
 	die ("ReadDMRaidPartitions(): dmsetup failed.");
 
     while (<DMDEV>) {
-	$dmdev = $_;
+	my $dmdev = $_;
 	chomp($dmdev);
 
 	#FIXME: I should not need to do this twice
 	if ($dmdev !~ m/part/) {
-            open (LOG, ">>$logname");
-            print LOG ("Find raid partition $dmdev\n");
-            close LOG;
+            $logger->milestone("Find raid partition $dmdev");
 	    # $dmdev is the base device
 	    $dmdev = "/dev/mapper/" . $dmdev;
 	    push @dmdisks, $dmdev;
 	}
 	#FIXME: need to check what needs to be removed
 	else {
-            open (LOG, ">>$logname");
-            print LOG ("Find raid disk $dmdev\n");
-            close LOG;
+            $logger->milestone("Find raid disk $dmdev");
 	    $dmdev = "/dev/mapper/" . $dmdev;
 	    push @dmparts, $dmdev;
 	}
@@ -558,18 +521,12 @@ sub ReadDMRaidPartitions {
     close DMDEV;
 
     my @devices = ();
-    my $dmpart;
-    my $tmp_part;
 
-    foreach $dmdev (@dmdisks) {
-	foreach $dmpart (@dmparts) {
+    foreach my $dmdev (@dmdisks) {
+	foreach my $dmpart (@dmparts) {
 	    if ($dmpart =~ m/$dmdev/) {
-		my $index = substr ($dmpart, length($dmpart)-2,2);
-
-		while (length ($index) > 0 && substr ($index, 0, 1) !~ /[0-9]/) {
-		    $index = substr ($index, 1);
-		}
-		push @devices, [$dmpart, $dmdev, $index];
+                $dmpart =~ m/^($dmdev)_part(\d+)$/;
+		push @devices, [$dmpart, $dmdev, $2];
 	    }
 	}
     }
@@ -616,15 +573,16 @@ returns 1 if yes, 0 if no
 =cut
 
 sub IsDMRaidSlave {
-
     my $disk = shift;
-    my $majmin_disk = Udev2MajMin($disk);
+
+    unless ( DMRaidAvailable() and -e $dmsetup) {
+        return 0;
+    }
+
+    my $majmin_disk = `stat -c "%t:%T" $disk`;
     chomp($majmin_disk);
     my @dmparts = ();
 
-    unless (-e $dmsetup) {
-        return 0;
-    }
 
     my @dm_devs = qx{$dmsetup info -c --noheadings -o name | grep -v part};
     chomp @dm_devs;
@@ -635,6 +593,7 @@ sub IsDMRaidSlave {
             chomp @tables;
 
             foreach my $line (@tables) {
+                next if $line !~ /mirror/;
                 my @content = split(/ /, $line);
 
                 foreach my $majmins (@content){
@@ -662,7 +621,7 @@ otherwise 0.
 sub IsDMDevice {
     my $dev = shift;
 
-    unless (-e $dmsetup) {
+    unless ( DMRaidAvailable() and -e $dmsetup) {
         return 0;
     }
 
@@ -677,170 +636,6 @@ sub IsDMDevice {
     return 0;
 }
 
-=item
-C<< $dm_dev = Bootloader::Tools::Udev2DMDev($udevdevice); >>
-
-takes a udev device (dm-X)
-returns the devicemapper device like dmsetup returns  
-
-=cut
-
-sub Bootloader::Tools::Udev2DMDev {
-
-    my $udevd = shift;
-    my $majmin = Udev2MajMin($udevd);
-    my $dmdev = MajMin2DMDev($majmin);
-
-    return $dmdev
-}
-
-=item
-C<< $udev_dev = Bootloader::Tools::DMDev2Udev($dmdev); >>
-
-takes a devicemapper device as reported from dmsetup
-returns a udev device (dm-X)
-
-=cut
-
-sub Bootloader::Tools::DMDev2Udev {
-
-    my $dmdev = shift;
-    my $majmin = DMDev2MajMin($dmdev);
-    my $udevdev = MajMin2Udev($majmin);
-
-    return $udevdev;
-}
-
-=item
-C<< $majmin = Bootloader::Tools::DMDev2MajMin($dmdev); >>
-
-takes a devicemapper device as reported from dmsetup
-returns a string containing major:minor
-
-=cut
-
-sub Bootloader::Tools::DMDev2MajMin {
-
-    my $dmdev = shift;
-    my $majmin;
-    $majmin =  qx{$dmsetup info -c  --noheadings -o major,minor '$dmdev'};
-    chomp $majmin;
-
-    return $majmin;
-}
-
-
-=item
-C<< $majmin = Bootloader::Tools::Udev2MajMin($udev); >>
-
-takes a udev device as reported from udevadm info
-returns a string containing major:minor
-
-=cut
-
-sub Bootloader::Tools::Udev2MajMin {
-
-    my $udev_dev = shift;
-    my $majmin;
-    my $cmd = "udevadm info -qpath -n $udev_dev";
-
-    if (my $udev_path = qx{$cmd 2>/dev/null}){
-       chomp ($udev_path);
-       
-       my $mounted = undef;
-       unless (-e "/sys/block") {
-         $mounted = `mount /sys`;
-       }
-       $majmin = qx{cat /sys$udev_path/dev};
-       chomp ($majmin);
-       `umount /sys` if (defined $mounted);
-
-    }
-    return $majmin;
-}
-
-=item
-C<< $udev_dev = Bootloader::Tools::MajMin2Udev($majmin); >>
-
-takes a string major:minor as reported from udevadm info
-returns a string containing the udev device as reported 
-by udevadm info
-
-=cut
-
-sub Bootloader::Tools::MajMin2Udev {
-
-    my $majmin = shift;
-
-    my ($major,  $minor) = split (/:/, $majmin );
-    my $udevdev;
-    my $sb="/sys/block";
-    my $devmajmin;
-
-    my $mounted = undef;
-    unless (-e "/sys/block") {
-      $mounted = `mount /sys`;
-    }
-    opendir (SYSBLOCK, "$sb");
-
-    foreach my $dirent (readdir(SYSBLOCK)) {
-	next if $dirent =~ m/^\./;
-	next if -r "$sb/$dirent/range" and qx{ cat $sb/$dirent/range } == 1;
-	$devmajmin = qx{cat $sb/$dirent/dev};
-        chomp $devmajmin;
-	my ($p, undef) = (split(/:/, $devmajmin));
-
-	if ("$p" eq "$major" ) {
-	    $udevdev = "$sb/$dirent";
-	    last;
-	}
-    }
-    closedir (SYSBLOCK);
-
-    opendir (SYSBLOCK, "$udevdev");
-    my $part;
-
-    foreach my $dirent (readdir(SYSBLOCK)) {
-	next if $dirent =~ m/^\./;
-
-	if (-r "$udevdev/$dirent/dev") {
-	    my $mm = qx{cat $udevdev/$dirent/dev};
-	    chomp ($mm);
-
-	    if ("$mm" eq "$majmin") {
-		$part = $dirent;
-		last;
-	    }
-	}
-    }
-    closedir (SYSBLOCK);
-
-    `umount /sys` if (defined $mounted);
-
-    return $part;
-}
-
-=item
-C<< $dm_dev = Bootloader::Tools::MajMin2DMDev($majmin); >>
-
-takes a string major:minor as reported from udevadm info
-returns a string containing the device as reported by 
-dmsetup
-
-=cut
-
-sub Bootloader::Tools::MajMin2DMDev {
-
-    my $majmin = shift;
-    my $dm_name  =  qx{devmap_name $majmin};
-    chomp $dm_name;
-
-    return $dm_name;
-
-}
-
-
-
 =item
 C<< $md_ref = Bootloader::Tools::ReadRAID1Arrays (); >>
 
@@ -851,6 +646,8 @@ to initialize the bootloader library properly.
 
 # FIXME: this has to be read through yast::storage
 sub ReadRAID1Arrays {
+    my $udevmapping = shift;
+    my $logger = Bootloader::Logger::instance();
     my %mapping = ();
     # use '/sbin/mdadm --detail --verbose --scan'
     # Assuming an output format like:
@@ -864,37 +661,43 @@ sub ReadRAID1Arrays {
     $mdadm = AddPathToExecutable("mdadm");
 
     if (-e $mdadm) {
-	open (MD, "$mdadm --detail --verbose --scan |");
+      open (MD, "$mdadm --detail --verbose --scan |");
     }
     else {
-	print ("The command \"mdadm\" is not available.\n");
-	print ("Is the package \"mdadm\" installed?\n");
+    	$logger->milestone ("The command \"mdadm\" is not available.");
+    	$logger->milestone ("Expect that system doesn't have MD array.");
 
-	# If the command "mdadm" isn't available, return a reference to an
-	# empty hash
-	return \%mapping;
+    	# If the command "mdadm" isn't available, return a reference to an
+    	# empty hash
+    	return \%mapping;
     }
-    
-    my $logname = Bootloader::Path::Logname();
-    qx{ $mdadm --detail --verbose --scan >> $logname};
 
     my ($array, $level, $num_devices);
+    $logger->milestone("Tools::ReadRAID1Arrays: start parsing mdadm --detail --verbose --scan:");
     while (my $line = <MD>)
     {
         chomp ($line);
+        $logger->milestone("Tools::ReadRAID1Arrays: $line");
 
         if ($line =~ /ARRAY (\S+) level=(\w+) num-devices=(\d+)/)
         {
             ($array, $level, $num_devices) = ($1, $2, $3);
+            #udevadm sometime return udev symlink instead of physical device, bnc#547580
+            $array = $udevmapping->{$array} if $udevmapping->{$array};
+            $logger->milestone("Tools::ReadRAID1Arrays: set array $array level $level and device count to $num_devices");
         }
         elsif ($level eq "raid1" and $line =~ /devices=(\S+)/)
         {
             # we could test $num_device against number of found devices to
             # detect degradedmode but that does not matter here (really?) 
-
-             $mapping{$array} = [ split(/,/, $1) ];
+             $logger->milestone("Tools::ReadRAID1Arrays: set to array $array values $1");
+      	     my @devices = split(/,/,$1);
+	           if ($devices[0]=~ m/^.*\d+$/){ #add only non-disc arrays
+      	       $mapping{$array} = [ @devices ];
+      	     }
         }
     }
+    $logger->milestone("Tools::ReadRAID1Arrays: finish parsing mdadm --detail --verbose --scan:");
     close( MD );
     return \%mapping;
 }
@@ -930,7 +733,7 @@ See AddSection for example
 sub GetSysconfigValue {
     my $key = shift;
     my $file = Bootloader::Path::Sysconfig();
-    return undef if ( qx{ grep -c ^[:space:]*$key $file} == 0);
+    return undef if ( qx{ grep -c ^[[:space:]]*$key $file} == 0);
     my $value = qx{ . $file && echo \$$key } || "";
     chomp ($value);
     return $value;
@@ -946,9 +749,10 @@ needed for it to run properly.
 
 sub InitLibrary {
     $lib_ref = Bootloader::Library->new ();
-    my $mp = ReadMountPoints ();
-    my $part = ReadPartitions ();
-    my $md = ReadRAID1Arrays ();
+    my $um = GetUdevMapping();
+    my $mp = ReadMountPoints ($um);
+    my $part = ReadPartitions ($um);
+    my $md = ReadRAID1Arrays ($um);
     my $mpath = GetMultipath ();
 
     $lib_ref->SetLoaderType (GetBootloader ());
@@ -956,11 +760,13 @@ sub InitLibrary {
     $lib_ref->DefinePartitions ($part);
     $lib_ref->DefineMDArrays ($md);
     $lib_ref->DefineMultipath ($mpath);
+    $lib_ref->DefineUdevMapping($um);
 
     # parse Bootloader configuration files   
     $lib_ref->ReadSettings();
 
     DumpLog ($lib_ref->{"loader"});
+    return $lib_ref;
 }
 
 
@@ -975,11 +781,10 @@ sub match_section {
 
     foreach my $opt (keys %{$opt_ref}) {
 	next unless exists $sect_ref->{"$opt"};
-	# FIXME: avoid "kernel"
 	# FIXME: if opt_ref doesn't have (hdX,Y), there is a mountpoint, thus remove it from sect_ref
         # FIXME: to compare !!
 	$core_lib->l_milestone ("Tools::match_section: matching key: $opt");
-	if ($opt eq "image" or $opt eq "kernel" or $opt eq "initrd") {
+	if ($opt eq "image" or $opt eq "initrd") {
 	    $match = (ResolveCrossDeviceSymlinks($sect_ref->{"$opt"}) eq
 		      $opt_ref->{"$opt"});
 	    # Print info for this match
@@ -1013,9 +818,6 @@ sub normalize_options {
 	    if exists $opt_ref->{"$_"};
 	$core_lib->l_milestone ("Tools::normalize_options: resolved result:" . $opt_ref->{"$_"});
     }
-
-    # FIXME: some have "kernel" some have "image" tag, latter makes more sense
-    $opt_ref->{"kernel"} = $opt_ref->{"image"} if exists $opt_ref->{"image"};
 }
 
 
@@ -1066,7 +868,9 @@ bootloader to use the current configuration
 
 
 sub UpdateBootloader {
-    $lib_ref->UpdateBootloader ();
+    my $ret = $lib_ref->UpdateBootloader ();
+    DumpLog ($lib_ref->{"loader"});
+    return $ret;
 }
 
 
@@ -1102,7 +906,7 @@ Get the default section, returns a hash reference
 EXAMPLE:
   my %section;
   %section = Bootloader::Tools::GetDefaultSection ();
-  my $default_kernel = $section{"kernel"};
+  my $default_kernel = $section{"image"};
 =cut
 
 sub GetDefaultSection {
@@ -1148,9 +952,7 @@ EXAMPLE:
 
 sub GetDefaultImage {
     my $ref = GetDefaultSection();
-    # FIXME: all modules under .../Core should use "image" as a key tag to the
-    # kernel image
-    return $ref->{"image"} || $ref->{"kernel"};
+    return $ref->{"image"};
 }
 
 =item
@@ -1198,9 +1000,16 @@ sub SetGlobals {
     }
     $glob_ref->{"__modified"} = 1;
     $lib_ref->SetGlobalSettings ($glob_ref);
-    $lib_ref->WriteSettings (1);
-    $lib_ref->UpdateBootloader (1); # avoid initialization but write config to
-                                    # the right place
+    unless (defined $lib_ref->WriteSettings (1)) {
+      $lib_ref->l_error ("Cannot write bootloader configuration file.");
+      DumpLog ($lib_ref->{"loader"});
+      return undef;
+    }
+    unless (defined $lib_ref->UpdateBootloader (1)) { # avoid initialization but write config to the right place
+      $lib_ref->l_error ("Cannot write bootloader configuration file.");
+      DumpLog ($lib_ref->{"loader"});
+      return undef;
+    }
     DumpLog ($lib_ref->{"loader"});
 }
 
@@ -1306,15 +1115,6 @@ sub AddSection {
 	}
     }
 
-    if (exists $option{"image"}) {
-	if (exists $new{"kernel"}) { 
-	    $new{"kernel"} = delete $option{"image"};
-	}
-	else {	
-	    $new{"image"} = delete $option{"image"};
-	}
-    }
-
     foreach (keys %option) {
 	$new{"$_"} = $option{"$_"};
     }
@@ -1333,14 +1133,17 @@ sub AddSection {
 	chomp ($arch);
 
 	if ($arch eq "i386") {
-	    $new{"append"} = "showopts ide=nodma apm=off acpi=off noresume nosmp noapic maxcpus=0 edd=off x11failsafe";
+	    $new{"append"} = "showopts ide=nodma apm=off acpi=off noresume nosmp noapic maxcpus=0 edd=off x11failsafe nomodeset";
 	}
 	elsif ($arch eq "x86_64") {
-	    $new{"append"} = "showopts ide=nodma apm=off acpi=off noresume edd=off x11failsafe";
+	    $new{"append"} = "showopts ide=nodma apm=off acpi=off noresume edd=off x11failsafe nomodeset";
 	}
 	elsif ($arch eq "ia64") {
 	    $new{"append"} = "ide=nodma nohalt noresume 3";
 	}
+	elsif ($arch eq "s390x") {
+	    $new{"append"} = "";
+  }
 	else {
 	    print ("Architecture $arch does not support failsafe entries.\n");
 	}
@@ -1357,12 +1160,6 @@ sub AddSection {
         $new{"append"} = $sysconf if (defined $sysconf);
         $sysconf = GetSysconfigValue("FAILSAFE_VGA");
         $new{"vgamode"} = $sysconf if (defined $sysconf);
-        $sysconf = GetSysconfigValue("IMAGEPCR");
-        $new{"imagepcr"} = $sysconf if (defined $sysconf);
-        $sysconf = GetSysconfigValue("INITRDPCR");
-        $new{"initrdpcr"} = $sysconf if (defined $sysconf);
-        my %measures = split(/:/, GetSysconfigValue("FAILSAFE_MEASURES"));
-        $new{"measure"} = \%measures if ((keys %measures)!= 0);
         $failsafe_modified = 1;
 	$default = 0;
     }
@@ -1374,32 +1171,27 @@ sub AddSection {
         $new{"vgamode"} = $sysconf if (defined $sysconf);
         $sysconf =  GetSysconfigValue("XEN_APPEND");
         $new{"xen_append"} =  $sysconf if (defined $sysconf);
-        $sysconf = GetSysconfigValue("XEN_IMAGEPCR");
-        $new{"imagepcr"} = $sysconf if (defined $sysconf);
-        $sysconf = GetSysconfigValue("XEN_INITRDPCR");
-        $new{"initrdpcr"} = $sysconf if (defined $sysconf);
-        $sysconf = GetSysconfigValue("XEN_PCR");
-        $new{"xenpcr"} = $sysconf if (defined $sysconf);
-        my %measures = split(/:/, GetSysconfigValue("XEN_MEASURES"));
-        $new{"measure"} = \%measures if ((keys %measures)!= 0);
     }
     else 
     {
-        $sysconf = GetSysconfigValue("DEFAULT_APPEND");
-        $new{"append"} = $sysconf if (defined $sysconf);
-        $sysconf = GetSysconfigValue("DEFAULT_VGA");
-        $new{"vgamode"} = $sysconf if (defined $sysconf);
-        $sysconf = GetSysconfigValue("IMAGEPCR");
-        $new{"imagepcr"} = $sysconf if (defined $sysconf);
-        $sysconf = GetSysconfigValue("INITRDPCR");
-        $new{"initrdpcr"} = $sysconf if (defined $sysconf);
-        my %measures = split(/:/, GetSysconfigValue("DEFAULT_MEASURES"));
-        $new{"measure"} = \%measures if ((keys %measures)!= 0);
-    }
+        #RT kernel have special args bnc #450153
+        if ($new{"image"} =~ m/-rt$/ && defined (GetSysconfigValue("RT_APPEND"))){
+          $sysconf = GetSysconfigValue("RT_APPEND");
+          $new{"append"} = $sysconf if (defined $sysconf);
+          $sysconf = GetSysconfigValue("RT_VGA");
+          $new{"vgamode"} = $sysconf if (defined $sysconf);
+        } else {
+          $sysconf = GetSysconfigValue("DEFAULT_APPEND");
+          $new{"append"} = $sysconf if (defined $sysconf);
+          $sysconf = GetSysconfigValue("DEFAULT_VGA");
+          $new{"vgamode"} = $sysconf if (defined $sysconf);
+        }
+     }
 
     $sysconf = GetSysconfigValue("CONSOLE");
     $new{"console"} = $sysconf if (defined $sysconf);
     $new{"__modified"} = 1;
+    $new{"root"} = $lib_ref->GetMountPoints()->{"/"};
 
     my $match = '';
     my $new_name = '';
@@ -1456,10 +1248,7 @@ sub AddSection {
 	# Delete obsolete (normal) boot entries from section array
 	my $section_index = 0;
 	foreach my $s (@sections) {
-	    if (exists $normal_entry->{"kernel"} && $normal_entry->{"kernel"} eq $s->{"kernel"}) {
-		delete $sections[$section_index];
-	    }
-	    elsif (exists $normal_entry->{"image"} && $normal_entry->{"image"} eq $s->{"image"}) {
+	    if (exists $normal_entry->{"image"} && $normal_entry->{"image"} eq $s->{"image"}) {
 		delete $sections[$section_index];
 	    }
 	    else {
@@ -1470,10 +1259,7 @@ sub AddSection {
 	# Delete obsolete (failsafe) boot entries from section array
 	$section_index = 0;
 	foreach my $s (@sections) {
-	    if (exists $failsafe_entry->{"kernel"} && $failsafe_entry->{"kernel"} eq $s->{"kernel"}) {
-		delete $sections[$section_index];
-	    }
-	    elsif (exists $failsafe_entry->{"image"} && $failsafe_entry->{"image"} eq $s->{"image"}) {
+	    if (exists $failsafe_entry->{"image"} && $failsafe_entry->{"image"} eq $s->{"image"}) {
 		delete $sections[$section_index];
 	    }
 	    else {
@@ -1539,10 +1325,16 @@ sub AddSection {
 			     } keys %{$glob_ref}) . "'\n"
 		       );
 
-    $lib_ref->WriteSettings (1);
-    $lib_ref->UpdateBootloader (1); # avoid initialization but write config to
-                                    # the right place
-
+    unless (defined $lib_ref->WriteSettings (1)) {
+      $core_lib->l_error ("Cannot write bootloader configuration file.");
+      DumpLog ($lib_ref->{"loader"});
+      return undef;
+    }
+    unless (defined $lib_ref->UpdateBootloader (1)) { # avoid initialization but write config to the right place
+      $core_lib->l_error ("Cannot write bootloader configuration file.");
+      DumpLog ($lib_ref->{"loader"});
+      return undef;
+    }
     DumpLog ($lib_ref->{"loader"});
 }
 
@@ -1691,9 +1483,16 @@ sub RemoveSections {
     $glob_ref->{"__modified"} = 1; # needed because of GRUB - index of default
 				   # may change even if not deleted
     $lib_ref->SetGlobalSettings ($glob_ref);
-    $lib_ref->WriteSettings (1);
-    $lib_ref->UpdateBootloader (1); # avoid initialization but write config to
-                                    # the right place
+    unless (defined $lib_ref->WriteSettings (1)) {
+      $core_lib->l_error ("Cannot write bootloader configuration file.");
+      DumpLog ($lib_ref->{"loader"});
+      return undef;
+    }
+    unless (defined $lib_ref->UpdateBootloader (1)) { # avoid initialization but write config to the right place
+      $core_lib->l_error ("Cannot write bootloader configuration file.");
+      DumpLog ($lib_ref->{"loader"});
+      return undef;
+    }
 
     DumpLog ($lib_ref->{"loader"});
 }
