@@ -4,7 +4,7 @@
 # Package:	Configuration of users and groups
 # Summary:	module for first stage user configuration
 #
-# $Id$
+# $Id: UsersSimple.pm 61042 2010-02-26 13:48:03Z jsuchome $
 #
 
 package UsersSimple;
@@ -81,6 +81,7 @@ my $cracklib_dictpath		= "";
 # if cracklib is used for password checking
 my $use_cracklib 		= 1;
 
+# this is yast2-users internal check now, feature was removed from PAM
 my $obscure_checks 		= 1;
 
 # User/group names must match the following regex expression. (/etc/login.defs)
@@ -115,6 +116,7 @@ YaST::YCP::Import ("Directory");
 YaST::YCP::Import ("FileUtils");
 YaST::YCP::Import ("Hostname");
 YaST::YCP::Import ("InstExtensionImage");
+YaST::YCP::Import ("Language");
 YaST::YCP::Import ("Mode");
 YaST::YCP::Import ("NetworkService");
 YaST::YCP::Import ("ProductControl");
@@ -613,12 +615,10 @@ sub CheckObscurity {
     if ($pw =~ m/$name/) {
 	if ($what eq "groups") {
 	    # popup question
-	    return __("You have used the group name as a part of the password.
-This is not a good security practice. Really use this password?");
+	    return __("You have used the group name as a part of the password.");
 	}
 	# popup question
-        return __("You have used the username as a part of the password.
-This is not a good security practice. Really use this password?");
+        return __("You have used the username as a part of the password.");
     }
 
     # check for lowercase
@@ -626,8 +626,7 @@ This is not a good security practice. Really use this password?");
     $filtered 		=~ s/[[:lower:]]//g;
     if ($filtered eq "") {
 	# popup question
-        return __("You have used only lowercase letters for the password.
-This is not a good security practice. Really use this password?");
+        return __("You have used only lowercase letters for the password.");
     }
 
     # check for uppercase
@@ -635,16 +634,14 @@ This is not a good security practice. Really use this password?");
     $filtered 		=~ s/[[:upper:]]//g;
     if ($filtered eq "") {
 	# popup question
-        return __("You have used only uppercase letters for the password.
-This is not a good security practice. Really use this password?");
+        return __("You have used only uppercase letters for the password.");
     }
     
     # check for palindroms
     $filtered 		= reverse $pw;
     if ($filtered eq $pw) {
 	# popup question
-        return __("You have used a palindrom for the password.
-This is not a good security practice. Really use this password?");
+        return __("You have used a palindrom for the password.");
     }
 
     # check for numbers
@@ -652,8 +649,7 @@ This is not a good security practice. Really use this password?");
     $filtered 		=~ s/[0-9]//g;
     if ($filtered eq "") {
 	# popup question
-        return __("You have used only digits for the password.
-This is not a good security practice. Really use this password?");
+        return __("You have used only digits for the password.");
     }
     return "";
 }
@@ -676,7 +672,7 @@ sub CheckPasswordMaxLength {
     if (length ($pw) > $max_length) {
 	# popup question
         $ret = sprintf (__("The password is too long for the current encryption method.
-Truncate it to %s characters?"), $max_length);
+It will be truncated to %s characters."), $max_length);
     }
     return $ret;
 }
@@ -734,72 +730,55 @@ Try again.");
     return "";
 }
 
-##------------------------------------
 # Check the password of given user or group: part 2, checking for
 # problems that may be skipped (accepted) by user
 # @param data map containing user/group name, password and type
-# @param answer map containing all the problems that were already skipped by
-# user
-# @return value is map with the problem found FIXME example
+#
+# Merges all error reports and returns them in the list
 BEGIN { $TYPEINFO{CheckPasswordUI} = ["function",
-    ["map", "string", "string"],
-    ["map", "string", "any"], ["map", "string", "any"]];
+    ["list", "string"],
+    ["map", "string", "any"]];
 }
 sub CheckPasswordUI {
 
-    my ($self, $data, $ui_map)	= @_;
+    my ($self, $data)	= @_;
     my $pw		= $data->{"userPassword"} || "";
     my $name		= $data->{"uid"};
     $name		= ($data->{"cn"} || "") if (!defined $name);
     my $type		= $data->{"type"} || "local";
     my $min_length 	= $self->GetMinPasswordLength ($type);
 
-    my %ret		= ();
+    my @ret		= ();
 
     if ($pw eq "") {
-	return \%ret;
+	return \@ret;
     }
 
-    if ($self->CrackLibUsed () && (($ui_map->{"crack"} || "") ne $pw)) {
+    if ($self->CrackLibUsed ()) {
 	my $error = $self->CrackPassword ($pw);
 	if ($error ne "") {
-	    $ret{"question_id"}	= "crack";
-	    # popup question
-	    $ret{"question"}	= sprintf (__("The password is too simple:
-%s
-Really use this password?"), $error);
-	    return \%ret;
+	    # error message
+	    push @ret, sprintf (__("The password is too simple:
+%s."), $error);
 	}
     }
     
-    if ($self->ObscureChecksUsed () && (($ui_map->{"obscure"} || "") ne $pw)) {
+    if (1) {#$self->ObscureChecksUsed ()) {
 	my $what	= "users";
 	$what		= "groups" if (! defined $data->{"uid"});
 	my $error	= $self->CheckObscurity ($name, $pw, $what);
-	if ($error ne "") {
-	    $ret{"question_id"}	= "obscure";
-	    $ret{"question"}	= $error;
-	    return \%ret;
-	}
+	push @ret, $error if $error;
     }
 
-    if (($ui_map->{"short"} || "") ne $pw) {
-	if (length ($pw) < $min_length) {
-	    $ret{"question_id"}	= "short";
-	    # popup questionm, %i is number
-	    $ret{"question"}	= sprintf (__("The password should have at least %i characters.
-Really use this shorter password?"), $min_length);
-	}
+    if (length ($pw) < $min_length) {
+	# popup error, %i is number
+	push @ret, sprintf (__("The password should have at least %i characters."), $min_length);
     }
     
-    if (($ui_map->{"truncate"} || "") ne $pw) {
-	my $error = $self->CheckPasswordMaxLength ($pw, $type);
-	if ($error ne "") {
-	    $ret{"question_id"}	= "truncate";
-	    $ret{"question"}	= $error;
-	}
-    }
-    return \%ret;
+    my $error = $self->CheckPasswordMaxLength ($pw, $type);
+    push @ret, $error if $error;
+
+    return \@ret;
 }
 
 ##------------------------------------
@@ -939,12 +918,16 @@ sub Write {
     }
     # enable inst_user to either run auth client or write first user
     if ($after_auth ne "users" || $user_defined) {
-	y2milestone ("enabling step 'user' for second stage");
+	# do not let both steps to run (bnc#672139)
 	if ($user_defined) {
+	    y2milestone ("enabling 'user_non_interactive', disabling 'user' for second stage");
 	    ProductControl->EnableModule ("user_non_interactive");
+	    ProductControl->DisableModule ("user");
 	}
 	else {
+	    y2milestone ("enabling 'user', disabling 'user_non_interactive' for second stage");
 	    ProductControl->EnableModule ("user");
+	    ProductControl->DisableModule ("user_non_interactive");
 	}
     }
     # no user entered + 2nd stage visible => enable clients (bnc#393722)
@@ -1225,6 +1208,13 @@ sub CheckNetworkMethodsAvailability {
 
     return $network_methods_checked if $network_methods_checked;
 
+    # no check during firstboot (bnc#458468)
+    if (Stage->firstboot ()) {
+	$network_methods_checked	= 1;
+	y2milestone ("firstboot stage: no check for network methods");
+	return 1;
+    }
+
     if (!NetworkService->isNetworkRunning()) {
 	y2milestone ("network is not running, skipping network methods test");
 	return 0;
@@ -1366,5 +1356,22 @@ sub UnLoadCracklib {
 	sformat (__("Releasing %1 extension..."), "cracklib-dict-full.rpm"));
 }
 
+##------------------------------------
+# use iconv transliteration feature to convert special characters to similar
+# ASCII ones (bnc#442225)
+BEGIN { $TYPEINFO{Transliterate} = ["function", "string", "string"]; }
+sub Transliterate {
+
+    my ($self, $text)	= @_;
+
+    return "" if ! $text;
+    my $language	= Language->language ();
+    my $out = SCR->Execute (".target.bash_output",
+	"echo '$text' | iconv -f utf-8 -t ascii//translit",
+	{ "LANG" => $language });
+    my $stdout = $out->{"stdout"} || "";
+    return $stdout;
+}
+
 42
 # EOF
